# Generated by Django 2.2.8 on 2020-01-07 14:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_discord_connector', '0007_auto_20191118_1402'),
    ]

    operations = [
        migrations.AlterField(
            model_name='discordgroup',
            name='group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='discord_group', to='auth.Group'),
        ),
    ]
